@@ -1,30 +1,36 @@
 from __future__ import annotations
 
+import asyncio
 import contextlib
 import warnings
 from collections.abc import AsyncGenerator
 from pathlib import Path
 from typing import Any
 
-from pydantic import SecretStr
-
 import kaos
 from kaos.path import KaosPath
+from kosong.message import ContentPart
+from pydantic import SecretStr
+
 from kimi_cli.agentspec import DEFAULT_AGENT_FILE
 from kimi_cli.cli import InputFormat, OutputFormat
 from kimi_cli.config import LLMModel, LLMProvider, load_config
 from kimi_cli.llm import augment_provider_with_env_vars, create_llm
 from kimi_cli.session import Session
 from kimi_cli.share import get_share_dir
-from kimi_cli.soul import LLMNotSet, LLMNotSupported
+from kimi_cli.soul import LLMNotSet, LLMNotSupported, run_soul
 from kimi_cli.soul.agent import Runtime, load_agent
 from kimi_cli.soul.context import Context
 from kimi_cli.soul.kimisoul import KimiSoul
 from kimi_cli.utils.logging import StreamToLogger, logger
 from kimi_cli.utils.path import shorten_home
+from kimi_cli.wire import Wire, WireUISide
+from kimi_cli.wire.message import WireMessage
 
 
 def enable_logging(debug: bool = False) -> None:
+    logger.remove()  # Remove default stderr handler
+    logger.enable("kimi_cli")
     if debug:
         logger.enable("kosong")
     logger.add(
@@ -101,7 +107,7 @@ class KimiCLI:
             agent_file = DEFAULT_AGENT_FILE
         agent = await load_agent(agent_file, runtime, mcp_configs=mcp_configs or [])
 
-        context = Context(session.history_file)
+        context = Context(session.context_file)
         await context.restore()
 
         soul = KimiSoul(agent, context=context)
@@ -132,7 +138,7 @@ class KimiCLI:
         return self._runtime.session
 
     @contextlib.asynccontextmanager
-    async def _app_env(self) -> AsyncGenerator[None]:
+    async def _env(self) -> AsyncGenerator[None]:
         original_cwd = KaosPath.cwd()
         await kaos.chdir(self._runtime.session.work_dir)
         try:
@@ -143,8 +149,58 @@ class KimiCLI:
         finally:
             await kaos.chdir(original_cwd)
 
-    async def run_shell_mode(self, command: str | None = None) -> bool:
-        from kimi_cli.ui.shell import ShellApp, WelcomeInfoItem
+    async def run(
+        self,
+        user_input: str | list[ContentPart],
+        cancel_event: asyncio.Event,
+        merge_wire_messages: bool = False,
+    ) -> AsyncGenerator[WireMessage]:
+        """
+        Run the Kimi CLI instance without any UI and yield Wire messages directly.
+
+        Args:
+            user_input (str | list[ContentPart]): The user input to the agent.
+            cancel_event (asyncio.Event): An event to cancel the run.
+            merge_wire_messages (bool): Whether to merge Wire messages as much as possible.
+
+        Yields:
+            WireMessage: The Wire messages from the `KimiSoul`.
+
+        Raises:
+            LLMNotSet: When the LLM is not set.
+            LLMNotSupported: When the LLM does not have required capabilities.
+            ChatProviderError: When the LLM provider returns an error.
+            MaxStepsReached: When the maximum number of steps is reached.
+            RunCancelled: When the run is cancelled by the cancel event.
+        """
+        async with self._env():
+            wire_future = asyncio.Future[WireUISide]()
+            stop_ui_loop = asyncio.Event()
+
+            async def _ui_loop_fn(wire: Wire) -> None:
+                wire_future.set_result(wire.ui_side(merge=merge_wire_messages))
+                await stop_ui_loop.wait()
+
+            soul_task = asyncio.create_task(
+                run_soul(self.soul, user_input, _ui_loop_fn, cancel_event)
+            )
+
+            try:
+                wire_ui = await wire_future
+                while True:
+                    msg = await wire_ui.receive()
+                    yield msg
+            except asyncio.QueueShutDown:
+                pass
+            finally:
+                # stop consuming Wire messages
+                stop_ui_loop.set()
+                # wait for the soul task to finish, or raise
+                await soul_task
+
+    async def run_shell(self, command: str | None = None) -> bool:
+        """Run the Kimi CLI instance with shell UI."""
+        from kimi_cli.ui.shell import Shell, WelcomeInfoItem
 
         welcome_info = [
             WelcomeInfoItem(
@@ -192,37 +248,40 @@ class KimiCLI:
                     level=WelcomeInfoItem.Level.INFO,
                 )
             )
-        async with self._app_env():
-            app = ShellApp(self._soul, welcome_info=welcome_info)
-            return await app.run(command)
+        async with self._env():
+            shell = Shell(self._soul, welcome_info=welcome_info)
+            return await shell.run(command)
 
-    async def run_print_mode(
+    async def run_print(
         self,
         input_format: InputFormat,
         output_format: OutputFormat,
         command: str | None = None,
     ) -> bool:
-        from kimi_cli.ui.print import PrintApp
+        """Run the Kimi CLI instance with print UI."""
+        from kimi_cli.ui.print import Print
 
-        async with self._app_env():
-            app = PrintApp(
+        async with self._env():
+            print_ = Print(
                 self._soul,
                 input_format,
                 output_format,
-                self._runtime.session.history_file,
+                self._runtime.session.context_file,
             )
-            return await app.run(command)
+            return await print_.run(command)
 
-    async def run_acp_server(self) -> bool:
-        from kimi_cli.ui.acp import ACPServer
+    async def run_acp(self) -> None:
+        """Run the Kimi CLI instance as ACP server."""
+        from kimi_cli.ui.acp import ACP
 
-        async with self._app_env():
-            app = ACPServer(self._soul)
-            return await app.run()
+        async with self._env():
+            acp = ACP(self._soul)
+            await acp.run()
 
-    async def run_wire_server(self) -> bool:
-        from kimi_cli.ui.wire import WireServer
+    async def run_wire_stdio(self) -> None:
+        """Run the Kimi CLI instance as Wire server over stdio."""
+        from kimi_cli.ui.wire import WireOverStdio
 
-        async with self._app_env():
-            server = WireServer(self._soul)
-            return await server.run()
+        async with self._env():
+            server = WireOverStdio(self._soul)
+            await server.serve()
