@@ -9,12 +9,15 @@ from typing import Annotated, Literal
 import typer
 
 from kimi_cli.constant import VERSION
+from kimi_cli.mcp import cli as mcp_cli
 
 
 class Reload(Exception):
     """Reload configuration."""
 
-    pass
+    def __init__(self, session_id: str | None = None):
+        super().__init__("reload")
+        self.session_id = session_id
 
 
 cli = typer.Typer(
@@ -34,8 +37,9 @@ def _version_callback(value: bool) -> None:
         raise typer.Exit()
 
 
-@cli.command()
+@cli.callback(invoke_without_command=True)
 def kimi(
+    ctx: typer.Context,
     version: Annotated[
         bool,
         typer.Option(
@@ -60,6 +64,13 @@ def kimi(
             help="Log debug information. Default: no.",
         ),
     ] = False,
+    agent: Annotated[
+        Literal["default", "okabe"] | None,
+        typer.Option(
+            "--agent",
+            help="Builtin agent specification to use. Default: builtin default agent.",
+        ),
+    ] = None,
     agent_file: Annotated[
         Path | None,
         typer.Option(
@@ -100,6 +111,14 @@ def kimi(
             help="Continue the previous session for the working directory. Default: no.",
         ),
     ] = False,
+    session_id: Annotated[
+        str | None,
+        typer.Option(
+            "--session",
+            "-S",
+            help="Session ID to resume for the working directory. Default: new session.",
+        ),
+    ] = None,
     command: Annotated[
         str | None,
         typer.Option(
@@ -194,29 +213,56 @@ def kimi(
     ] = None,
 ):
     """Kimi, your next CLI agent."""
+    if ctx.invoked_subcommand is not None:
+        return  # skip rest if a subcommand is invoked
+
     del version  # handled in the callback
 
-    from kaos import current_kaos
-    from kaos.local import LocalKaos
     from kaos.path import KaosPath
+
+    from kimi_cli.agentspec import DEFAULT_AGENT_FILE, OKABE_AGENT_FILE
     from kimi_cli.app import KimiCLI, enable_logging
-    from kimi_cli.metadata import WorkDirMeta, load_metadata, save_metadata
+    from kimi_cli.mcp import get_global_mcp_config_file
+    from kimi_cli.metadata import load_metadata, save_metadata
     from kimi_cli.session import Session
     from kimi_cli.utils.logging import logger
 
     enable_logging(debug)
 
-    special_flags = {
-        "--print": print_mode,
-        "--acp": acp_mode,
-        "--wire": wire_mode,
-    }
-    active_specials = [flag for flag, active in special_flags.items() if active]
-    if len(active_specials) > 1:
-        raise typer.BadParameter(
-            f"Cannot combine {', '.join(active_specials)}.",
-            param_hint=active_specials[0],
-        )
+    if session_id is not None:
+        session_id = session_id.strip()
+        if not session_id:
+            raise typer.BadParameter("Session ID cannot be empty", param_hint="--session")
+
+    conflict_option_sets = [
+        {
+            "--print": print_mode,
+            "--acp": acp_mode,
+            "--wire": wire_mode,
+        },
+        {
+            "--agent": agent is not None,
+            "--agent-file": agent_file is not None,
+        },
+        {
+            "--continue": continue_,
+            "--session": session_id is not None,
+        },
+    ]
+    for option_set in conflict_option_sets:
+        active_options = [flag for flag, active in option_set.items() if active]
+        if len(active_options) > 1:
+            raise typer.BadParameter(
+                f"Cannot combine {', '.join(active_options)}.",
+                param_hint=active_options[0],
+            )
+
+    if agent is not None:
+        match agent:
+            case "default":
+                agent_file = DEFAULT_AGENT_FILE
+            case "okabe":
+                agent_file = OKABE_AGENT_FILE
 
     ui: UIMode = "shell"
     if print_mode:
@@ -245,6 +291,12 @@ def kimi(
     file_configs = list(mcp_config_file or [])
     raw_mcp_config = list(mcp_config or [])
 
+    # Use default MCP config file if no MCP config is provided
+    if not file_configs:
+        default_mcp_file = get_global_mcp_config_file()
+        if default_mcp_file.exists():
+            file_configs.append(default_mcp_file)
+
     try:
         mcp_configs = [json.loads(conf.read_text(encoding="utf-8")) for conf in file_configs]
     except json.JSONDecodeError as e:
@@ -255,12 +307,18 @@ def kimi(
     except json.JSONDecodeError as e:
         raise typer.BadParameter(f"Invalid JSON: {e}", param_hint="--mcp-config") from e
 
-    async def _run() -> bool:
-        work_dir = (
-            KaosPath.unsafe_from_local_path(local_work_dir) if local_work_dir else KaosPath.cwd()
-        )
+    work_dir = KaosPath.unsafe_from_local_path(local_work_dir) if local_work_dir else KaosPath.cwd()
 
-        if continue_:
+    async def _run(session_id: str | None) -> bool:
+        if session_id is not None:
+            session = await Session.find(work_dir, session_id)
+            if session is None:
+                raise typer.BadParameter(
+                    f"No session with id {session_id} found for the working directory",
+                    param_hint="--session",
+                )
+            logger.info("Switching to session: {session_id}", session_id=session.id)
+        elif continue_:
             session = await Session.continue_(work_dir)
             if session is None:
                 raise typer.BadParameter(
@@ -271,7 +329,6 @@ def kimi(
         else:
             session = await Session.create(work_dir)
             logger.info("Created new session: {session_id}", session_id=session.id)
-        logger.debug("Session history file: {history_file}", history_file=session.history_file)
 
         if thinking is None:
             metadata = load_metadata()
@@ -289,9 +346,9 @@ def kimi(
         )
         match ui:
             case "shell":
-                succeeded = await instance.run_shell_mode(command)
+                succeeded = await instance.run_shell(command)
             case "print":
-                succeeded = await instance.run_print_mode(
+                succeeded = await instance.run_print(
                     input_format or "text",
                     output_format or "text",
                     command,
@@ -299,27 +356,26 @@ def kimi(
             case "acp":
                 if command is not None:
                     logger.warning("ACP server ignores command argument")
-                succeeded = await instance.run_acp_server()
+                await instance.run_acp()
+                succeeded = True
             case "wire":
                 if command is not None:
                     logger.warning("Wire server ignores command argument")
-                succeeded = await instance.run_wire_server()
+                await instance.run_wire_stdio()
+                succeeded = True
 
         if succeeded:
             metadata = load_metadata()
 
             # Update work_dir metadata with last session
-            work_dir_meta = next(
-                (wd for wd in metadata.work_dirs if wd.path == str(session.work_dir)), None
-            )
+            work_dir_meta = metadata.get_work_dir_meta(session.work_dir)
 
             if work_dir_meta is None:
                 logger.warning(
                     "Work dir metadata missing when marking last session, recreating: {work_dir}",
                     work_dir=session.work_dir,
                 )
-                work_dir_meta = WorkDirMeta(path=str(session.work_dir))
-                metadata.work_dirs.append(work_dir_meta)
+                work_dir_meta = metadata.new_work_dir_meta(session.work_dir)
 
             work_dir_meta.last_session_id = session.id
 
@@ -330,17 +386,29 @@ def kimi(
 
         return succeeded
 
-    current_kaos.set(LocalKaos())
     while True:
         try:
-            succeeded = asyncio.run(_run())
-            if succeeded:
-                break
-            raise typer.Exit(code=1)
-        except Reload:
+            succeeded = asyncio.run(_run(session_id))
+            session_id = None
+            if not succeeded:
+                raise typer.Exit(code=1)
+            break
+        except Reload as e:
+            session_id = e.session_id
             continue
 
 
+@cli.command()
+def acp():
+    """Run Kimi CLI ACP server."""
+    from kimi_cli.acp import acp_main
+
+    acp_main()
+
+
+cli.add_typer(mcp_cli, name="mcp")
+
+
 if __name__ == "__main__":
     if "kimi_cli.cli" not in sys.modules:
         sys.modules["kimi_cli.cli"] = sys.modules[__name__]
