@@ -10,9 +10,9 @@ from datetime import datetime
 from pathlib import Path
 from typing import Any
 
+from kaos.path import KaosPath
 from kosong.tooling import Toolset
 
-from kaos.path import KaosPath
 from kimi_cli.agentspec import load_agent_spec
 from kimi_cli.config import Config
 from kimi_cli.llm import LLM
@@ -21,6 +21,7 @@ from kimi_cli.soul.approval import Approval
 from kimi_cli.soul.denwarenji import DenwaRenji
 from kimi_cli.soul.toolset import KimiToolset, ToolType
 from kimi_cli.tools import SkipThisTool
+from kimi_cli.utils.environment import Environment
 from kimi_cli.utils.logging import logger
 from kimi_cli.utils.path import list_directory
 
@@ -63,6 +64,7 @@ class Runtime:
     denwa_renji: DenwaRenji
     approval: Approval
     labor_market: LaborMarket
+    environment: Environment
 
     @staticmethod
     async def create(
@@ -71,9 +73,10 @@ class Runtime:
         session: Session,
         yolo: bool,
     ) -> Runtime:
-        ls_output, agents_md = await asyncio.gather(
+        ls_output, agents_md, environment = await asyncio.gather(
             list_directory(session.work_dir),
             load_agents_md(session.work_dir),
+            Environment.detect(),
         )
 
         return Runtime(
@@ -89,6 +92,7 @@ class Runtime:
             denwa_renji=DenwaRenji(),
             approval=Approval(yolo=yolo),
             labor_market=LaborMarket(),
+            environment=environment,
         )
 
     def copy_for_fixed_subagent(self) -> Runtime:
@@ -101,6 +105,7 @@ class Runtime:
             denwa_renji=DenwaRenji(),  # subagent must have its own DenwaRenji
             approval=self.approval,
             labor_market=LaborMarket(),  # fixed subagent has its own LaborMarket
+            environment=self.environment,
         )
 
     def copy_for_dynamic_subagent(self) -> Runtime:
@@ -113,6 +118,7 @@ class Runtime:
             denwa_renji=DenwaRenji(),  # subagent must have its own DenwaRenji
             approval=self.approval,
             labor_market=self.labor_market,  # dynamic subagent shares LaborMarket with main agent
+            environment=self.environment,
         )
 
 
@@ -190,6 +196,7 @@ async def load_agent(
         DenwaRenji: runtime.denwa_renji,
         Approval: runtime.approval,
         LaborMarket: runtime.labor_market,
+        Environment: runtime.environment,
     }
     tools = agent_spec.tools
     if agent_spec.exclude_tools:
@@ -257,14 +264,16 @@ def _load_tool(tool_path: str, dependencies: dict[type[Any], Any]) -> ToolType |
     if cls is None:
         return None
     args: list[type[Any]] = []
-    for param in inspect.signature(cls).parameters.values():
-        if param.kind == inspect.Parameter.KEYWORD_ONLY:
-            # once we encounter a keyword-only parameter, we stop injecting dependencies
-            break
-        # all positional parameters should be dependencies to be injected
-        if param.annotation not in dependencies:
-            raise ValueError(f"Tool dependency not found: {param.annotation}")
-        args.append(dependencies[param.annotation])
+    if "__init__" in cls.__dict__:
+        # the tool class overrides the `__init__` of base class
+        for param in inspect.signature(cls).parameters.values():
+            if param.kind == inspect.Parameter.KEYWORD_ONLY:
+                # once we encounter a keyword-only parameter, we stop injecting dependencies
+                break
+            # all positional parameters should be dependencies to be injected
+            if param.annotation not in dependencies:
+                raise ValueError(f"Tool dependency not found: {param.annotation}")
+            args.append(dependencies[param.annotation])
     return cls(*args)
 
 
@@ -283,6 +292,11 @@ async def _load_mcp_tools(
     from kimi_cli.tools.mcp import MCPTool
 
     for mcp_config in mcp_configs:
+        # Skip empty MCP configs (no servers defined)
+        if not mcp_config.get("mcpServers"):
+            logger.debug("Skipping empty MCP config: {mcp_config}", mcp_config=mcp_config)
+            continue
+
         logger.info("Loading MCP tools from: {mcp_config}", mcp_config=mcp_config)
         client = fastmcp.Client(mcp_config)
         async with client:
