@@ -61,6 +61,10 @@ class Context:
     def n_checkpoints(self) -> int:
         return self._next_checkpoint_id
 
+    @property
+    def file_backend(self) -> Path:
+        return self._file_backend
+
     async def checkpoint(self, add_user_message: bool):
         checkpoint_id = self._next_checkpoint_id
         self._next_checkpoint_id += 1
@@ -92,14 +96,14 @@ class Context:
             logger.error("Checkpoint {checkpoint_id} does not exist", checkpoint_id=checkpoint_id)
             raise ValueError(f"Checkpoint {checkpoint_id} does not exist")
 
-        # rotate the history file
+        # rotate the context file
         rotated_file_path = await next_available_rotation(self._file_backend)
         if rotated_file_path is None:
             logger.error("No available rotation path found")
             raise RuntimeError("No available rotation path found")
         await aiofiles.os.replace(self._file_backend, rotated_file_path)
         logger.debug(
-            "Rotated history file: {rotated_file_path}", rotated_file_path=rotated_file_path
+            "Rotated context file: {rotated_file_path}", rotated_file_path=rotated_file_path
         )
 
         # restore the context until the specified checkpoint
@@ -127,6 +131,33 @@ class Context:
                     message = Message.model_validate(line_json)
                     self._history.append(message)
 
+    async def clear(self):
+        """
+        Clear the context history.
+        This is almost equivalent to revert_to(0), but without relying on the assumption
+        that the first checkpoint exists.
+        File backend will be rotated.
+
+        Raises:
+            RuntimeError: When no available rotation path is found.
+        """
+
+        logger.debug("Clearing context")
+
+        # rotate the context file
+        rotated_file_path = await next_available_rotation(self._file_backend)
+        if rotated_file_path is None:
+            logger.error("No available rotation path found")
+            raise RuntimeError("No available rotation path found")
+        await aiofiles.os.replace(self._file_backend, rotated_file_path)
+        logger.debug(
+            "Rotated context file: {rotated_file_path}", rotated_file_path=rotated_file_path
+        )
+
+        self._history.clear()
+        self._token_count = 0
+        self._next_checkpoint_id = 0
+
     async def append_message(self, message: Message | Sequence[Message]):
         logger.debug("Appending message(s) to context: {message}", message=message)
         messages = message if isinstance(message, Sequence) else [message]
