@@ -13,8 +13,7 @@ class FileActions(str, Enum):
 
 
 from .glob import Glob  # noqa: E402
-from .grep import Grep  # noqa: E402
-from .patch import PatchFile  # noqa: E402
+from .grep_local import Grep  # noqa: E402
 from .read import ReadFile  # noqa: E402
 from .replace import StrReplaceFile  # noqa: E402
 from .write import WriteFile  # noqa: E402
@@ -25,5 +24,4 @@ __all__ = (
     "Grep",
     "WriteFile",
     "StrReplaceFile",
-    "PatchFile",
 )
