@@ -12,7 +12,7 @@ from kosong.message import (
     ThinkPart,
     ToolCall,
 )
-from rich.console import Group
+from rich.console import Group, RenderableType
 from rich.panel import Panel
 from rich.rule import Rule
 from rich.syntax import Syntax
@@ -23,7 +23,7 @@ from kimi_cli.ui.shell.console import console
 from kimi_cli.ui.shell.metacmd import meta_command
 
 if TYPE_CHECKING:
-    from kimi_cli.ui.shell import ShellApp
+    from kimi_cli.ui.shell import Shell
 
 
 def _format_content_part(part: ContentPart) -> Text | Panel | Group:
@@ -108,14 +108,11 @@ def _format_message(msg: Message, index: int) -> Panel:
         role_text += f" [dim]→ {msg.tool_call_id}[/dim]"
 
     # Format content
-    content_items: list = []
+    content_items: list[RenderableType] = []
 
-    if isinstance(msg.content, str):
-        content_items.append(Text(msg.content, style="white"))
-    else:
-        for part in msg.content:
-            formatted = _format_content_part(part)
-            content_items.append(formatted)
+    for part in msg.content:
+        formatted = _format_content_part(part)
+        content_items.append(formatted)
 
     # Add tool calls if present
     if msg.tool_calls:
@@ -144,11 +141,11 @@ def _format_message(msg: Message, index: int) -> Panel:
 
 
 @meta_command(kimi_soul_only=True)
-def debug(app: ShellApp, args: list[str]):
+def debug(app: Shell, args: list[str]):
     """Debug the context"""
     assert isinstance(app.soul, KimiSoul)
 
-    context = app.soul._context
+    context = app.soul.context
     history = context.history
 
     if not history:
@@ -168,7 +165,7 @@ def debug(app: ShellApp, args: list[str]):
                 Text(f"Total messages: {len(history)}", style="bold"),
                 Text(f"Token count: {context.token_count:,}", style="bold"),
                 Text(f"Checkpoints: {context.n_checkpoints}", style="bold"),
-                Text(f"Trajectory: {context._file_backend}", style="dim"),
+                Text(f"Trajectory: {context.file_backend}", style="dim"),
             ),
             title="[bold]Context Info[/bold]",
             border_style="cyan",
