@@ -9,13 +9,12 @@ from dataclasses import dataclass
 from kosong.message import Message, TextPart
 from kosong.tooling import ToolError, ToolOk
 
-from kimi_cli.soul import StatusSnapshot
 from kimi_cli.ui.shell.console import console
 from kimi_cli.ui.shell.prompt import PROMPT_SYMBOL
 from kimi_cli.ui.shell.visualize import visualize
-from kimi_cli.utils.message import message_extract_text, message_stringify
+from kimi_cli.utils.message import message_stringify
 from kimi_cli.wire import Wire
-from kimi_cli.wire.message import ContentPart, StepBegin, ToolCall, ToolResult
+from kimi_cli.wire.message import ContentPart, StatusUpdate, StepBegin, ToolCall, ToolResult
 
 MAX_REPLAY_RUNS = 5
 
@@ -45,7 +44,7 @@ async def replay_recent_history(history: Sequence[Message]) -> None:
         wire = Wire()
         console.print(f"{getpass.getuser()}{PROMPT_SYMBOL} {message_stringify(run.user_message)}")
         ui_task = asyncio.create_task(
-            visualize(wire.ui_side, initial_status=StatusSnapshot(context_usage=0.0))
+            visualize(wire.ui_side(merge=False), initial_status=StatusUpdate(context_usage=None))
         )
         for event in run.events:
             wire.soul_side.send(event)
@@ -59,7 +58,7 @@ def _is_user_message(message: Message) -> bool:
     # FIXME: should consider non-text tool call results which are sent as user messages
     if message.role != "user":
         return False
-    return not message_extract_text(message).startswith("<system>CHECKPOINT")
+    return not message.extract_text().startswith("<system>CHECKPOINT")
 
 
 def _find_replay_start(history: Sequence[Message]) -> int | None:
@@ -84,23 +83,22 @@ def _build_replay_runs(history: Sequence[Message]) -> list[_ReplayRun]:
                 continue
             current_run.n_steps += 1
             current_run.events.append(StepBegin(n=current_run.n_steps))
-            if isinstance(message.content, str):
-                current_run.events.append(TextPart(text=message.content))
-            else:
-                current_run.events.extend(message.content)
+            current_run.events.extend(message.content)
             current_run.events.extend(message.tool_calls or [])
         elif message.role == "tool":
             if current_run is None:
                 continue
             assert message.tool_call_id is not None
-            if isinstance(message.content, list) and any(
+            if any(
                 isinstance(part, TextPart) and part.text.startswith("<system>ERROR")
                 for part in message.content
             ):
                 result = ToolError(message="", output="", brief="")
             else:
                 result = ToolOk(output=message.content)
-            current_run.events.append(ToolResult(tool_call_id=message.tool_call_id, result=result))
+            current_run.events.append(
+                ToolResult(tool_call_id=message.tool_call_id, return_value=result)
+            )
     if current_run is not None:
         runs.append(current_run)
     return runs
