@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from pathlib import Path
 from typing import NamedTuple
 
@@ -85,7 +87,7 @@ def parse_changelog(md_text: str) -> dict[str, ReleaseEntry]:
     return result
 
 
-def format_release_notes(changelog: dict[str, ReleaseEntry]) -> str:
+def format_release_notes(changelog: dict[str, ReleaseEntry], include_lib_changes: bool) -> str:
     parts: list[str] = []
     for ver, entry in changelog.items():
         s = f"[bold]{ver}[/bold]"
@@ -93,6 +95,8 @@ def format_release_notes(changelog: dict[str, ReleaseEntry]) -> str:
             s += f": {entry.description}"
         if entry.entries:
             for it in entry.entries:
+                if it.lower().startswith("lib:") and not include_lib_changes:
+                    continue
                 s += "\n[markdown.item.bullet]• [/]" + it
         parts.append(s + "\n")
     return "\n".join(parts).strip()
